/**
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2004 satoshi akabane(akabane@logical-paradox.org)
 *
 */
package org.logical_paradox.console;

import java.io.IOException;
import java.io.OutputStream;

import javax.swing.JTextArea;

/**
 * hLgǗNX
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.7 $
 */
public class Document {
	/** vvg */
	private String prompt;
	/** vvg̃TCY */
	private int promptLength;

	/** eLXgR|[lg */
	private JTextArea textarea;
	/** P[VJE^ */
	private int ptr;
	/** J[\Xʒu(_s) */
	private int xpos;
	private boolean inputmode = false;

	/** ENTERꂽۂ̍s */
	private String commitedLine;

	/** o̓Xg[ */
	private final OutputStream ostream;

	/**
	 * RXgN^
	 * @param text eLXgGA
	 */
	public Document(JTextArea text) {
		textarea = text;
		xpos = ptr = 0;
		ostream = new DocumentOutputStream(this);
		prompt = "";
		promptLength = 0;
	}
	/**
	 * RXgN^
	 * @param text eLXgGA
	 * @param ppt vvg
	 */
	public Document(JTextArea text, String ppt) {
		this(text);
		if(ppt != null) {
			prompt = ppt;
			promptLength = ppt.length();
		}
	}
	/**
	 * ̃hLgɑ΂o̓Xg[Ԃ
	 * @return ̃hLgɘAĂo̓Xg[
	 */
	public OutputStream getOutputStream() {
		return ostream;
	}
	/**
	 * ݂̃eLXgԂ
	 * @return eLXg
	 */
	public String getText() {
		return textarea.getText();
	}
	/**
	 * J[\Cӂ̈ʒuֈړ
	 * @param pos eLXgł̃J[\̈ʒu
	 */
	public void moveCursor(int pos) {
		textarea.setCaretPosition(pos);
		ptr = pos;
	}
	/**
	 * J[\݂̌̈ʒuĒ
	 */
	public void relocate() {
		moveCursor(ptr);
	}
	public boolean isJapaneseMode() {
		return inputmode;
	}
	public void turnInputMode() {
	}
	/**
	 * ݂̃J[\ʒuփeLXg}
	 * @param msg }eLXg
	 */
	public void insert(String msg) {
		// J[\\OFF
		textarea.getCaret().setVisible(false);
		textarea.insert(msg, ptr);
		// J[\ړ
		ptr += msg.length();
		xpos += msg.length();
		// J[\\ON
		textarea.getCaret().setVisible(true);
	}
	/**
	 * ݂̃J[\ʒu֕}
	 * @param c
	 */
	public void insert(char c) {
		insert("" + c);
	}
	/**
	 * ss܂ł폜
	 */
	public void deleteLine() {
		moveToLineBegin();
		textarea.setText(textarea.getText().substring(0, ptr));
	}
	/**
	 * J[\sֈړ
	 */
	public void moveToLineBegin() {
		// xŌ̍s̍Ō̕ֈړ
		moveToEnd();
		int p = getLineBeginPos();
		xpos = promptLength;
		moveCursor(p);
		relocate();
	}
	/**
	 * J[\sֈړ
	 */
	public void moveToEnd() {
		textarea.setCaretPosition(textarea.getText().length());
		ptr = textarea.getText().length();
		xpos = getCurrentXPos();
	}
	/**
	 * J[\E1ړ
	 */
	public void moveToRight() {
		// słȂꍇ͈ړł
		if(
			ptr < textarea.getText().length() &&
			textarea.getText().charAt(ptr) != '\n'
		) {
			ptr++;
			xpos++;
			relocate();
		}
	}
	/**
	 * J[\1ړ
	 */
	public void moveToLeft() {
		// słȂΈړł
		if(xpos > promptLength) {
			ptr--;
			xpos--;
			relocate();
		}
	}
	/**
	 * s
	 */
	public void CRLF() {
		moveToEnd();
		insert('\n');
		xpos = 0;
	}
	/**
	 * sm肷
	 */
	public void commit() {
		moveToEnd();
		setCommitedLine(getCurrentLine());
		CRLF();
	}
	/**
	 * vvgo͂
	 */
	public void prompt() {
		insert(prompt);
	}
	/**
	 * ݂̃J[\ʒuԂ
	 * @return J[\ʒu
	 */
	public int getCurrentPosition() {
		return ptr;
	}
	/**
	 * ݂̃J[\ʒu̕폜
	 * DELL[ƓԂɂȂ
	 */
	public void delete() {
		if(ptr < textarea.getText().length()) {
			String text = textarea.getText();
			textarea.setText(text.substring(0, ptr) + text.substring(ptr+1));
		}

		relocate();
	}
	/**
	 * ݂̃J[\ʒu1O̕폜CJ[\ړ
	 * BSL[ƓԂɂȂ
	 */
	public void backSpace() {
		// J[\ړ
		// Cs̏ꍇ͈ړȂ
		if(xpos > promptLength) {
			String text = textarea.getText();
			textarea.setText(
				text.substring(0, ptr-1) +
				(ptr >= text.length() ? "" : text.substring(ptr))
			);
			xpos--;
			ptr--;

			relocate();
		}
	}
	/**
	 * sT
	 * @return s̈ʒu
	 */
	public int getLineBeginPos() {
		int bXpos = xpos;
		int bPtr = ptr;
		while(bXpos > promptLength) {
			bXpos--;
			bPtr--;
		}

		return bPtr;
	}
	/**
	 * ݂̃J[\XWvZ
	 * @return J[\XW
	 */
	protected int getCurrentXPos() {
		int bPtr = ptr;
		int xpos = 0;
		while(bPtr > promptLength && textarea.getText().charAt(bPtr-1) != '\n') {
			bPtr--;
			xpos++;
		}

		return xpos;
	}
	/**
	 * ŌENTERꂽۂ̍seݒ肷
	 * @return se
	 */
	protected void setCommitedLine(String line) {
		commitedLine = line;
	}
	/**
	 * ŌENTERꂽۂ̍seԂ
	 * @return se
	 */
	public String getCommitedLine() {
		return commitedLine == null ? "" : commitedLine;
	}
	/**
	 * ݃J[\̂s̓eԂ
	 * @return se
	 */
	protected String getCurrentLine() {
		return textarea.getText().substring(getLineBeginPos());
	}
	/**
	 * hLgNXɘAo̓Xg[
	 * @author satoshi akabane@logical-paradox.org
	 * @version $Revision: 1.7 $
	 */
	class DocumentOutputStream extends OutputStream {
		/** hLg */
		private final Document doc;
		/**
		 * RXgN^
		 * @param d hLg
		 */
		protected DocumentOutputStream(Document d) {
			doc = d;
		}
		/**
		 * obt@ɑ΂1oCgo͂
		 * @param b o͑Ώ
		 * @throws IOException o͂Ɏsꍇ̗O
		 */
		public void write(int b) throws IOException {
			if(b == '\n') {
				doc.CRLF();
			} else {
				doc.insert((char)b);
			}
		}
	}
}
